#undef NDEBUG
#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <list>
#include <map>
#include <string>

#include "GeometryModifyTool.hpp"
#include "OCCModifyEngine.hpp"
#include "Body.hpp"
#include "RefEntity.hpp"
#include "InitCGMA.hpp"
#include "CubitCompat.hpp"

#define STRINGIFY(S) XSTRINGIFY(S)
#define XSTRINGIFY(S) #S
#ifndef SRCDIR
#  define SRCDIR "."
#endif

using std::list;
using std::pair;
using std::set;
using std::map;
using std::vector;

int main( int argc, char** argv ) {

  CubitStatus status = InitCGMA::initialize_cgma("OCC");
  if (CUBIT_SUCCESS != status) return 1;
  GeometryModifyTool *gmti = GeometryModifyTool::instance();   
  
  Body* box = gmti->brick(1,1,1);
  Body* sphere = gmti->sphere(0.7);

  DLIList<Body*> from_bodies, new_bodies;
  from_bodies.append(sphere);
 
  CubitStatus rsl = gmti->intersect(box, from_bodies, new_bodies, CUBIT_TRUE);
  if(rsl == CUBIT_FAILURE)
    return 1;

  int num_ents_exported=0;
  const CubitString cubit_version="10.2";
  const char * filename = "intersect.brep";
  const char * filetype = "OCC";
  DLIList<RefEntity*> ref_entity_list;
  CAST_LIST(new_bodies, ref_entity_list, RefEntity);

  rsl = CubitCompat_export_solid_model(ref_entity_list, filename, filetype,
                                 num_ents_exported, cubit_version);

  ref_entity_list.clean_out();
  new_bodies.clean_out();
  DLIList<Body*> tool_bodies;
  tool_bodies.append(box);
  rsl = gmti->subtract(tool_bodies, from_bodies, new_bodies, false, true);
  filename = "sphere-box.brep";
  CAST_LIST(new_bodies, ref_entity_list, RefEntity);

  rsl = CubitCompat_export_solid_model(ref_entity_list, filename, filetype,
                                 num_ents_exported, cubit_version);

  ref_entity_list.clean_out();
  new_bodies.clean_out();
  rsl = gmti->subtract(from_bodies, tool_bodies, new_bodies);
  filename = "box-sphere.brep";
  CAST_LIST(new_bodies, ref_entity_list, RefEntity);  

  rsl = CubitCompat_export_solid_model(ref_entity_list, filename, filetype,
                                 num_ents_exported, cubit_version);
  printf("done...\n");

  return(0);

}
